/**
 * SWARM 1.0.0
 * 
 *
 * Copyright 2014, Kate Kligman
 * All rights reserved.
 */

function parseColor(color)
{
    var r, g, b;

    r = parseInt(color.substr(1, 2), 16);
    g = parseInt(color.substr(3, 2), 16);
    b = parseInt(color.substr(5, 2), 16);

    return {r: r, g: g, b: b};
}

function formatColor(rgb)
{
    return '#' + prependZeros(2, rgb['r'].toString(16)) + prependZeros(2, rgb['g'].toString(16)) + prependZeros(2, rgb['b'].toString(16));
}

function prependZeros(amount, str) {
    str = str.toString();
    while (str.length < amount) {
        str = '0' + str;
    }
    return str;
}

function lerpColor(start, end, percent)
{
    var s, e, n;

    s = parseColor(start);
    e = parseColor(end);

    n = {};
    n.r = Math.floor(Crafty.math.lerp(s.r, e.r, percent));
    n.g = Math.floor(Crafty.math.lerp(s.g, e.g, percent));
    n.b = Math.floor(Crafty.math.lerp(s.b, e.b, percent));

    return formatColor(n);
}
Crafty.scene("game", function()
{
    // Backgrounds
    Crafty.background("#000000");
    Crafty.e("2D, Canvas, Image").image("img/starsbg.png").attr({x: 0, y: 0});

    Crafty.e("Canvas, Laser");

    // Top planet
    Crafty.e("2D, Canvas, Image").image("img/mars.png").attr({x: 0, y: 0});

    // Bottom planet. When the swarm reaches the planet, the game is over.
    Crafty.e("2D, Canvas, Image, Collision")
        .image("img/earth.png")
        .attr({x: 0, y: Crafty.viewport.height - 17})
        .onHit('ship1', function() {
            Crafty.trigger('GameOver');
            swarmGame.doGameOver();
            Crafty("Jelly").trigger("Freeze");
        });

    // Top right score
    Crafty.e('Score');

    // When the user clicks in the game window, fire the laser.
    Crafty.e("mouseTracking, 2D, Mouse, Touch, Canvas")
        .attr({ w:Crafty.viewport.width, h:Crafty.viewport.height, x:0, y:0 })
        .bind('MouseDown', function(e) {
            if (swarmGame.gameState.gameOver === false) {
                Crafty.trigger('Fire', {x: Crafty.mousePos.x, y: Crafty.mousePos.y});
                Crafty.audio.play('laser' + Crafty.math.randomInt(1,2));
                swarmGame.gameState.stats.shots++;
            }
        });

    Crafty.audio.play('start');

    // Enemy spawner
    var spawner = Crafty.e('Spawner');
    spawner.trigger('Start');
});
Crafty.c("Jelly", {
    setLevel: function(level) {
        this._level = level;

        switch (this._level) {
            case 1:
                this._yspeed = 80;
                this._xspeed = 0;
                this._xslide = 0;
                break;
            case 2:
                this._yspeed = 100;
                this._xspeed = 0;
                this._xslide = 0;
                break;
            case 3:
                this._yspeed = 120;
                this._xspeed = 50;
                this._xslide = 100;
                break;
            case 4:
                this._yspeed = 130;
                this._xspeed = 50;
                this._xslide = 100;
        }
    },
    init: function() {

        var gs = swarmGame.gameState;
        this.setLevel(1);

        this.addComponent("2D, Canvas, ship1, animate, Mouse, Delay");
        this.origin("center");

        this.attr({
            x: Crafty.math.randomInt(50, swarmGame.spawnWidth - 50),
            y: Crafty.math.randomInt(35, 3),
            w: 32, // width
            h: 32  // height
        });

        this.xorig = this.x;
        this.xdir = Crafty.math.randomElementOfArray([-1, 1]);

        this.bind("EnterFrame", function(frame) {
            if (gs.gameOver == true)
                return;

            if (Crafty.math.abs(this.xorig - this.x) > this._xslide) {
                this.xdir *= -1;
            }

            this.x += (this._xspeed * (frame.dt/1000) * this.xdir);
            this.y += this._yspeed * (frame.dt/1000);

        });

        // Show a smiley face
        this.bind('GameOver', function() {
            this.removeComponent('ship1').addComponent('ship4');
        });

        // Ship is hit with the laser
        this.bind('Click', function(e) {
            var gs = swarmGame.gameState;

            if (gs.gameOver == true)
                return;

            Crafty.trigger('Fire', {x: Crafty.mousePos.x, y: Crafty.mousePos.y});
            gs.stats.shots++;

            if (this.has("ship1")) {
                this.removeComponent("ship1").addComponent("ship2");
                Crafty.trigger('ScoreUpdate', 100 * gs.level);
                Crafty.audio.play('laser' + Crafty.math.randomInt(1,2));
                Crafty.audio.play('explosion');

                gs.stats.kills++;
                if (gs.stats.kills % 10 == 0)
                    gs.level++;

                this.delay(function() {
                    this.destroy();
                }, 200);
            }
        });
    }
});
Crafty.c("Laser", {

    init: function() {

        this.addComponent('2D');
        this.addComponent('Tint');

        // Create a second canvas to act as an overlay to draw the laser on.
        // Canvas init and cleanup largely from particles code in crafty.js.
        var canvas = document.createElement("canvas");
        canvas.width = Crafty.viewport.width;
        canvas.height = Crafty.viewport.height;
        canvas.style.position = 'absolute';
        canvas.style.top = '0px';
        canvas.style.left = '0px';
        Crafty.stage.elem.appendChild(canvas);

        this.ctx = canvas.getContext('2d');

        // Remove the second canvas from the DOM when the laser is removed.
        this.bind('Remove', function () {
            Crafty.stage.elem.removeChild(canvas);
        }).bind("RemoveComponent", function (id) {
            if (id === 'Laser')
                Crafty.stage.elem.removeChild(canvas);
        });

        this.laser_stage = 'off';
        this.laser_duration = 0;
        this.laser_max_duration = 100; // ms

        this.w = canvas.width;
        this.h = canvas.height;

        this.color = '#FFFF00';
        this.radius = 20;

        this.x = 0;
        this.y = 0;

        this.laser_previous_stage = 'off';

        this.laserActive = true;

        this.bind("EnterFrame", this.behavior);
        this.bind("Fire", this.fire);
    },

    fire: function(data) {
        //if (this.laserActive !== true)
            //return;
        this.laser_stage = 'on';
        this.mx = data.x;
        this.my = data.y;
    },

    activate: function(setting) {
        this.laserActive = setting;
    },

    behavior: function(frame) {
        this.laser_previous_stage = this.laser_stage;

        if (this.laser_stage == 'on') {
            // If the laser has exceeded its maximum display duration, shut it down.
            if (this.laser_duration > this.laser_max_duration) {
                this.laser_stage = 'off';
                this.laser_duration = 0;
                this.color = '#FFFF00';
                // Otherwise: show the laser effect, computing the current color as a function of display time.
            } else {
                //this.color = shadeColor(this.color, (this.laser_duration) / this.laser_max_duration);
                //this.color = lerpColor('#FF0000', '#FFFFFF', this.laser_duration / this.laser_max_duration);
                this.color = lerpColor('#FFFF00', '#FFFFFF', this.laser_duration / this.laser_max_duration);
                this.laser_duration += frame.dt;
            }
            // Will eventually call draw() to render laser changes for the next frame.
            this.trigger("Invalidate");
        }
    },

    draw: function() {

        // If the laser was previously on, clear the canvas for next time.
        if (this.laser_previous_stage == 'on' && this.laser_stage == 'off') {
            this.ctx.clearRect(0, 0, this.ctx.canvas.width, this.ctx.canvas.height);
        }

        // Do nothing if the laser isn't firing yet.
        if (this.laser_stage == 'off')
            return;

        var ctx = this.ctx;

        ctx.save();
        ctx.lineWidth = 2;
        ctx.strokeStyle = this.color;
        ctx.beginPath();
        ctx.moveTo(0, ctx.canvas.height);
        ctx.lineTo(this.mx, this.my);
        ctx.stroke();
        ctx.moveTo(ctx.canvas.width, ctx.canvas.height);
        ctx.lineTo(this.mx, this.my);
        ctx.stroke();
        ctx.closePath();

    }
});
var swarmGame = {
    viewportWidth: 960,
    viewportHeight: 540,
    spawnWidth: 960 - 50,
    spawnHeight: 540 - 50,
    gameState: {
        level: 1,
        gameOver: true,
        stats: {
            kills: 0,
            shots: 0,
            time: 0,
            wins: 0
        }
    },
    init: function() {
        Crafty.init(this.viewportWidth, this.viewportHeight, "game");
        Crafty.canvas.init();
    },
    resetState: function() {
        var gs = this.gameState;
        gs.level = 1;
        gs.stats.kills = 0;
        gs.stats.shots = 0;
        gs.stats.time = 0;
        gs.stats.wins = 0;
        gs.gameOver = false;

    },
    resetGame: function() {
        swarmGame.resetState();
        Crafty("Jelly").destroy();
        Crafty.scene("game");
    },
    doGameOver: function() {
        var gs = swarmGame.gameState;
        var accuracy = 0;

        gs.gameOver = true;
        Crafty('Jelly').trigger('GameOver');

        Crafty.e("GameOverBg, 2D, DOM, Image, Mouse")
            .image("img/gameover.png")
            .attr({x: 250, y: 150})
            .bind("Click", function() {
                this.destroy();
                swarmGame.resetGame();
            });

        Crafty.e("Stats1, 2D, DOM, Text, Color")
            .attr({ x: 440, y: 280, w: 200, h: 30 })
            .textFont({size: '16px', family: 'Arial', weight: 'bold'})
            .textColor('blue', 16)
            .text("Kills: " + gs.stats.kills);

        Crafty.e("Stats1, 2D, DOM, Text, Color")
            .attr({ x: 440, y: 300, w: 200, h: 30 })
            .textFont({size: '16px', family: 'Arial', weight: 'bold'})
            .textColor('blue', 16)
            .text("Shots Fired: " + gs.stats.shots);

        if (gs.stats.shots > 0)
            accuracy = Math.round((gs.stats.kills/gs.stats.shots) * 100);

        Crafty.e("Stats1, 2D, DOM, Text, Color")
            .attr({ x: 440, y: 320, w: 200, h: 30 })
            .textFont({size: '16px', family: 'Arial', weight: 'bold'})
            .textColor('blue', 16)
            .text("Accuracy: " + accuracy + ' %');
    }
}

swarmGame.init();


// Asset loader

Crafty.load(
    [
        "img/earth.png",
        "img/mars.png",
        "img/starsbg.png",
        "img/gameover.png",
        "snd/explosion.wav",
        "snd/start.wav",
        "snd/laser1.wav",
        "snd/laser2.wav"
    ],
    // On Load complete
    function() {

        Crafty.audio.add({
            start: [
                "snd/start.wav"
            ],
            laser1: [
                "snd/laser1.wav"
            ],
            laser2: [
                "snd/laser2.wav"
            ],
            explosion: [
                "snd/explosion.wav"
            ]
        });

        Crafty.sprite(32, "img/ship1.png", {
            ship1: [0, 0],
            ship2: [1, 0],
            ship3: [2, 0],
            ship4: [3, 0]
        });



        // Start the game
        swarmGame.resetGame();
    },

    // Load in progress
    function(e) {
    },
    // Load error
    function(e) {
    }
);


/*

if (window.innerWidth < 960) {
    viewportWidth = Crafty.DOM.window.width;
    viewportHeight = Crafty.DOM.window.height;
} else {
    viewportWidth = 960;
    viewportHeight = 540;
//}

*/
Crafty.c('Score', {
   init: function() {
       this.addComponent('2D, Canvas, Color, Text');
       this.attr({x: 820, y: 20, w: 130, score: 0})
           .textFont({size: '16px', family: 'Arial', weight: 'bold'})
           .textColor('#FFFFFF', 8)
           .color('#000000')
           .bind('ScoreUpdate', function(score) {
               this.score = this.score + score;
               this.text('Score: ' + this.score)
           })
           .text('Score: 0');
   }
});
Crafty.c("Spawner", {
    // ["Entity", level, quantity, spawn delay, spawn start offset]
    waves: [
        [
            ["Jelly", 1, 5, 1000, 0],
            ["Jelly", 2, 5, 1000, 5000],

        ],
        [
            ["Jelly", 1, 10, 1000, 0],
            ["Jelly", 2, 5, 1000, 5000]
        ],
        [
            ["Jelly", 1, 5, 800, 0],
            ["Jelly", 1, 5, 800, 0]
        ],
        [
            ["Jelly", 1, 5, 500, 0],
            ["Jelly", 2, 7, 700, 0]
        ],
        [
            ["Jelly", 1, 5, 3000, 0],
            ["Jelly", 2, 10, 1500, 0],
            ["Jelly", 3, 10, 1000, 0]
        ],
    ],

    schedule: function(level) {
        var levelTime = 0;

        if (!this.spawning)
            return;

        if (level >= this.waves.length)
            level = this.waves.length - 1;

        this.levelItems = 0;

        for (var wave = 0; wave < this.waves[level].length; wave++) {
            var item = this.waves[level][wave];
            this.levelItems += item[2];
            if (item[2] * item[3] + item[4] > levelTime)
                levelTime = item[2] * item[3] + item[4];
            this.postpone()(this, item);

        }
    },
    postpone: function() {
        return function(instance, item) {
            instance.sleep(function() {
                console.log("Awaking from sleep to spawn");
                instance.delay(function() {
                    if (this.spawning) {
                        var e = Crafty.e(item[0]);
                        e.setLevel(item[1]);
                    }
                }, item[3], item[2]);
            }, item[4]);
        }
    },
    init: function() {
        this.addComponent("Delay");
        this.level = 0;
        this.spawning = true;

        this.bind('Start', function() {
            this.levelItems = 0;
            this.schedule(this.level);
        });

        this.bind('Stop', function() {
            this.spawning = false;
            console.log('Stop');
        });

        this.bind('GameOver', function() {
            this.spawning = false;
        });

        this.bind('Fire', function() {
            this.levelItems--;
            if (0 === this.levelItems) {
                console.log("End of Level: " + this.level);
                this.sleep(function() {
                    this.level++;
                    if (this.spawning)
                        this.schedule(this.level);
                }, 5000);
            }
        });
    },

    sleep: function(callback, interval) {
        this.delay(function() {
            callback.call(this);
        }, interval);
    }
});
Crafty.c('Stats', {
    init: function() {

    }
});