Crafty.c("Laser", {

    init: function() {

        this.addComponent('2D');
        this.addComponent('Tint');

        // Create a second canvas to act as an overlay to draw the laser on.
        // Canvas init and cleanup largely from particles code in crafty.js.
        var canvas = document.createElement("canvas");
        canvas.width = Crafty.viewport.width;
        canvas.height = Crafty.viewport.height;
        canvas.style.position = 'absolute';
        canvas.style.top = '0px';
        canvas.style.left = '0px';
        Crafty.stage.elem.appendChild(canvas);

        this.ctx = canvas.getContext('2d');

        // Remove the second canvas from the DOM when the laser is removed.
        this.bind('Remove', function () {
            Crafty.stage.elem.removeChild(canvas);
        }).bind("RemoveComponent", function (id) {
            if (id === 'Laser')
                Crafty.stage.elem.removeChild(canvas);
        });

        this.laser_stage = 'off';
        this.laser_duration = 0;
        this.laser_max_duration = 100; // ms

        this.w = canvas.width;
        this.h = canvas.height;

        this.color = '#FFFF00';
        this.radius = 20;

        this.x = 0;
        this.y = 0;

        this.laser_previous_stage = 'off';

        this.laserActive = true;

        this.bind("EnterFrame", this.behavior);
        this.bind("Fire", this.fire);
    },

    fire: function(data) {
        //if (this.laserActive !== true)
            //return;
        this.laser_stage = 'on';
        this.mx = data.x;
        this.my = data.y;
    },

    activate: function(setting) {
        this.laserActive = setting;
    },

    behavior: function(frame) {
        this.laser_previous_stage = this.laser_stage;

        if (this.laser_stage == 'on') {
            // If the laser has exceeded its maximum display duration, shut it down.
            if (this.laser_duration > this.laser_max_duration) {
                this.laser_stage = 'off';
                this.laser_duration = 0;
                this.color = '#FFFF00';
                // Otherwise: show the laser effect, computing the current color as a function of display time.
            } else {
                //this.color = shadeColor(this.color, (this.laser_duration) / this.laser_max_duration);
                //this.color = lerpColor('#FF0000', '#FFFFFF', this.laser_duration / this.laser_max_duration);
                this.color = lerpColor('#FFFF00', '#FFFFFF', this.laser_duration / this.laser_max_duration);
                this.laser_duration += frame.dt;
            }
            // Will eventually call draw() to render laser changes for the next frame.
            this.trigger("Invalidate");
        }
    },

    draw: function() {

        // If the laser was previously on, clear the canvas for next time.
        if (this.laser_previous_stage == 'on' && this.laser_stage == 'off') {
            this.ctx.clearRect(0, 0, this.ctx.canvas.width, this.ctx.canvas.height);
        }

        // Do nothing if the laser isn't firing yet.
        if (this.laser_stage == 'off')
            return;

        var ctx = this.ctx;

        ctx.save();
        ctx.lineWidth = 2;
        ctx.strokeStyle = this.color;
        ctx.beginPath();
        ctx.moveTo(0, ctx.canvas.height);
        ctx.lineTo(this.mx, this.my);
        ctx.stroke();
        ctx.moveTo(ctx.canvas.width, ctx.canvas.height);
        ctx.lineTo(this.mx, this.my);
        ctx.stroke();
        ctx.closePath();

    }
});